@@ -12,7 +12,7 @@ def sleep_print(sec: int):
 def main():
     setup_logger()
 
-    tasks = [random.randint(0, 100) for i in range(100000)]
+    tasks = [random.randint(0, 100) for _ in range(10000)]
 
     client = DaskClient("127.0.0.1:12345")
 
