@@ -16,7 +16,7 @@ def main():
 
     address = "tcp://127.0.0.1:2345"
 
-    cluster = SchedulerClusterCombo(address=address, n_workers=10, per_worker_queue_size=2, event_loop="uvloop")
+    cluster = SchedulerClusterCombo(address=address, n_workers=10, per_worker_queue_size=2, event_loop="builtin")
     client = Client(address=address)
 
     tasks = [random.randint(0, 101) for _ in range(10000)]
