@@ -6,8 +6,9 @@ import threading
 import uuid
 from collections import defaultdict
 from concurrent.futures import Future
+from inspect import signature
 
-from typing import Callable, Dict, List, Optional, Tuple
+from typing import Any, Callable, Dict, List, Optional, Tuple
 
 import zmq
 
@@ -25,6 +26,7 @@ from scaled.protocol.python.message import (
     MonitorRequest,
     MonitorResponse,
     Task,
+    TaskCancel,
     TaskEcho,
     TaskEchoStatus,
     TaskResult,
@@ -46,6 +48,7 @@ class Client:
             bind_or_connect="connect",
             address=ZMQConfig.from_string(address),
             callback=self.__on_receive,
+            exit_callback=self.__on_exit,
             daemonic=True,
         )
         self._connector.start()
@@ -67,7 +70,9 @@ class Client:
         function_id, function_bytes = self.__get_function_id(fn)
 
         task_id = uuid.uuid1().bytes
-        task = Task(task_id, function_id, self._serializer.serialize_arguments(args, kwargs))
+        all_args = Client.__convert_kwargs_to_args(fn, args, kwargs)
+
+        task = Task(task_id, function_id, self._serializer.serialize_arguments(all_args))
         self._task_id_to_task_function[task_id] = (task, function_bytes)
 
         self.__on_buffer_task_send(task)
@@ -105,7 +110,7 @@ class Client:
         raise TypeError(f"Unknown {message_type=}")
 
     def __on_task_echo(self, task_echo: TaskEcho):
-        if task_echo.task_id not in self._task_id_to_future:
+        if task_echo.task_id not in self._task_id_to_task_function:
             return
 
         if task_echo.status == TaskEchoStatus.Duplicated:
@@ -121,8 +126,8 @@ class Client:
 
         assert task_echo.status == TaskEchoStatus.SubmitOK, f"Unknown task status: " f"{task_echo=}"
 
-        future = self._task_id_to_future[task_echo.task_id]
-        future.set_running_or_notify_cancel()
+        self._task_id_to_task_function.pop(task_echo.task_id)
+        self._task_id_to_future[task_echo.task_id].set_running_or_notify_cancel()
 
     def __on_buffer_task_send(self, task):
         if task.function_id not in self._function_id_to_not_ready_tasks:
@@ -146,9 +151,7 @@ class Client:
         if result.task_id not in self._task_id_to_future:
             return
 
-        self._task_id_to_task_function.pop(result.task_id)
         future = self._task_id_to_future.pop(result.task_id)
-
         if result.status == TaskStatus.Success:
             future.set_result(self._serializer.deserialize_result(result.result))
             return
@@ -157,12 +160,17 @@ class Client:
             future.set_exception(pickle.loads(result.result))
             return
 
-        if result.status == TaskStatus.Canceled:
-            return
-
     def __on_statistics_response(self, monitor_response: MonitorResponse):
         self._statistics_future.set_result(monitor_response.data)
 
+    def __on_exit(self):
+        if not self._task_id_to_future:
+            return
+
+        logging.info(f"canceling {len(self._task_id_to_future)} tasks")
+        for task_id in self._task_id_to_future.keys():
+            self._connector.send_immediately(MessageType.TaskCancel, TaskCancel(task_id))
+
     def __get_function_id(self, fn: Callable) -> Tuple[bytes, bytes]:
         if fn in self._function_to_function_id_cache:
             return self._function_to_function_id_cache[fn]
@@ -175,3 +183,33 @@ class Client:
         function_bytes = self._serializer.serialize_function(fn)
         function_id = hashlib.md5(function_bytes).digest()
         return function_id, function_bytes
+
+    @staticmethod
+    def __convert_kwargs_to_args(fn: Callable, args: Tuple[Any], kwargs: Dict[str, Any]) -> Tuple:
+        all_params = [p for p in signature(fn).parameters.values()]
+
+        keyword_only_arguments = tuple(p.name for p in all_params if p.kind == p.KEYWORD_ONLY)
+        if keyword_only_arguments:
+            raise TypeError(f"client doesn't support {fn} has keyword only arguments: {keyword_only_arguments}")
+
+        params = [p for p in all_params if p.kind in {p.POSITIONAL_ONLY, p.POSITIONAL_OR_KEYWORD}]
+
+        if len(args) >= len(params):
+            return args
+
+        number_of_required = len([p for p in params if p.default is p.empty])
+
+        args = list(args)
+        kwargs = kwargs.copy()
+
+        for p in params[len(args) : number_of_required]:
+            try:
+                args.append(kwargs.pop(p.name))
+            except KeyError:
+                missing = tuple(p.name for p in params[len(args) : number_of_required])
+                raise TypeError(f"{fn} missing {len(missing)} arguments: {missing}")
+
+        for p in params[len(args) :]:
+            args.append(kwargs.pop(p.name, p.default))
+
+        return tuple(args)
