@@ -18,6 +18,7 @@ class SchedulerProcess(multiprocessing.get_context("spawn").Process):
         worker_timeout_seconds: int,
         function_retention_seconds: int,
         load_balance_seconds: int,
+        load_balance_trigger_times: int,
         event_loop: Literal["builtin", "uvloop"] = "builtin",
     ):
         multiprocessing.Process.__init__(self, name="Scheduler")
@@ -28,10 +29,12 @@ class SchedulerProcess(multiprocessing.get_context("spawn").Process):
         self._worker_timeout_seconds = worker_timeout_seconds
         self._function_retention_seconds = function_retention_seconds
         self._load_balance_seconds = load_balance_seconds
+        self._load_balance_trigger_times = load_balance_trigger_times
 
         self._event_loop = event_loop
         self._scheduler: Optional[Scheduler] = None
         self._loop = None
+        self._task = None
 
     def run(self) -> None:
         # scheduler have its own single process
@@ -44,11 +47,11 @@ class SchedulerProcess(multiprocessing.get_context("spawn").Process):
             worker_timeout_seconds=self._worker_timeout_seconds,
             function_retention_seconds=self._function_retention_seconds,
             load_balance_seconds=self._load_balance_seconds,
+            load_balance_trigger_times=self._load_balance_trigger_times,
         )
 
         register_event_loop(self._event_loop)
 
         self._loop = asyncio.get_event_loop()
-        for coroutine in self._scheduler.get_loops():
-            self._loop.create_task(coroutine())
-        self._loop.run_forever()
+        self._task = self._loop.create_task(self._scheduler.get_loops())
+        self._loop.run_until_complete(self._scheduler.get_loops())
