@@ -1,11 +1,9 @@
-import asyncio
 import logging
 from collections import defaultdict
 import time
 from typing import Dict, Optional, Set
 
 from scaled.io.async_binder import AsyncBinder
-from scaled.io.config import CLEANUP_INTERVAL_SECONDS
 from scaled.protocol.python.message import (
     FunctionRequest,
     FunctionRequestType,
@@ -81,16 +79,7 @@ class VanillaFunctionManager(FunctionManager, Looper):
         if not task_ids:
             self._function_id_to_task_ids.pop(function_id)
 
-    async def loop(self):
-        logging.info(f"{self.__class__.__name__}: started")
-        while True:
-            await self.__routine()
-            await asyncio.sleep(CLEANUP_INTERVAL_SECONDS)
-
-    async def statistics(self) -> Dict:
-        return {"function_id_to_tasks": {k.hex(): len(v) for k, v in self._function_id_to_task_ids.items()}}
-
-    async def __routine(self):
+    async def routine(self):
         now = time.time()
         dead_functions = [
             function_id
@@ -105,6 +94,9 @@ class VanillaFunctionManager(FunctionManager, Looper):
             )
             self._function_id_to_alive_since.pop(function_id)
 
+    async def statistics(self) -> Dict:
+        return {"function_id_to_tasks": {k.hex(): len(v) for k, v in self._function_id_to_task_ids.items()}}
+
     async def __on_function_check(self, client: bytes, function_id: bytes):
         if function_id in self._function_id_to_function:
             await self.__send_function_response(client, function_id, FunctionResponseType.OK)
