@@ -1,8 +1,11 @@
+import asyncio
 import json
 import logging
 
+from scaled.io.config import CLEANUP_INTERVAL_SECONDS
 from scaled.scheduler.client_manager.vanilla import VanillaClientManager
 from scaled.scheduler.function_manager.vanilla import VanillaFunctionManager
+from scaled.utility.event_loop import create_async_loop_routine
 from scaled.utility.zmq_config import ZMQConfig
 from scaled.io.async_binder import AsyncBinder
 from scaled.protocol.python.message import MessageType, MessageVariant, MonitorRequest, MonitorResponse
@@ -20,6 +23,7 @@ class Scheduler:
         worker_timeout_seconds: int,
         function_retention_seconds: int,
         load_balance_seconds: int,
+        load_balance_trigger_times: int,
     ):
         self._address = address
 
@@ -31,6 +35,7 @@ class Scheduler:
             per_worker_queue_size=per_worker_queue_size,
             timeout_seconds=worker_timeout_seconds,
             load_balance_seconds=load_balance_seconds,
+            load_balance_trigger_times=load_balance_trigger_times,
         )
 
         self._binder.register(self.on_receive_message)
@@ -56,7 +61,11 @@ class Scheduler:
             return
 
         if message_type == MessageType.TaskCancel:
-            await self._task_manager.on_task_cancel(source, message.task_id)
+            await self._task_manager.on_task_cancel(source, message)
+            return
+
+        if message_type == MessageType.TaskCancelEcho:
+            await self._worker_manager.on_task_cancel_echo(source, message)
             return
 
         if message_type == MessageType.TaskResult:
@@ -69,8 +78,17 @@ class Scheduler:
 
         logging.error(f"{self.__class__.__name__}: unknown {message_type} from {source=}: {message}")
 
-    def get_loops(self):
-        return [self._binder.loop, self._task_manager.loop, self._function_manager.loop, self._worker_manager.loop]
+    async def get_loops(self):
+        try:
+            await asyncio.gather(
+                create_async_loop_routine(self._binder.routine, 0),
+                create_async_loop_routine(self._task_manager.routine, 0),
+                create_async_loop_routine(self._function_manager.routine, CLEANUP_INTERVAL_SECONDS),
+                create_async_loop_routine(self._worker_manager.routine, CLEANUP_INTERVAL_SECONDS),
+                return_exceptions=True,
+            )
+        except asyncio.CancelledError:
+            pass
 
     async def statistics(self, source: bytes, request: MonitorRequest):
         assert isinstance(request, MonitorRequest)
