@@ -1,12 +1,11 @@
-import asyncio
-from asyncio import Queue
 import logging
-from typing import Dict, Literal, Optional, Set, Tuple
+from typing import Dict, Optional, Set
 
 from scaled.io.async_binder import AsyncBinder
 from scaled.protocol.python.message import (
     MessageType,
     Task,
+    TaskCancel,
     TaskCancelEcho,
     TaskEcho,
     TaskEchoStatus,
@@ -14,6 +13,7 @@ from scaled.protocol.python.message import (
     TaskStatus,
 )
 from scaled.scheduler.mixins import FunctionManager, Looper, TaskManager, WorkerManager
+from scaled.utility.queues.async_indexed_queue import IndexedQueue
 
 
 class VanillaTaskManager(TaskManager, Looper):
@@ -26,10 +26,10 @@ class VanillaTaskManager(TaskManager, Looper):
         self._task_id_to_client: Dict[bytes, bytes] = dict()
         self._task_id_to_task: Dict[bytes, Task] = dict()
 
+        self._unassigned: IndexedQueue[bytes] = IndexedQueue()
         self._running: Set[bytes] = set()
-        self._canceling: Set[bytes] = set()
-        self._unassigned: Queue[Tuple[bytes, Task]] = Queue()
 
+        self._success_count: int = 0
         self._failed_count: int = 0
         self._canceled_count: int = 0
 
@@ -38,17 +38,20 @@ class VanillaTaskManager(TaskManager, Looper):
         self._function_manager = function_manager
         self._worker_manager = worker_manager
 
-    async def loop(self):
-        logging.info(f"{self.__class__.__name__}: started")
-        while True:
-            await self.__routine()
-            await asyncio.sleep(0)
+    async def routine(self):
+        task_id = await self._unassigned.get()
+
+        if not await self._worker_manager.assign_task_to_worker(self._task_id_to_task[task_id]):
+            await self._unassigned.put(task_id)
+            return
+
+        self._running.add(task_id)
 
     async def statistics(self) -> Dict:
         return {
-            "running": len(self._running),
-            "canceling": len(self._canceling),
             "unassigned": self._unassigned.qsize(),
+            "running": len(self._running),
+            "success": self._success_count,
             "failed": self._failed_count,
             "canceled": self._canceled_count,
         }
@@ -69,65 +72,53 @@ class VanillaTaskManager(TaskManager, Looper):
 
         await self._binder.send(client, MessageType.TaskEcho, TaskEcho(task.task_id, TaskEchoStatus.SubmitOK))
         await self._function_manager.on_task_use_function(task.task_id, task.function_id)
-        await self._unassigned.put((client, task))
+        self._task_id_to_client[task.task_id] = client
+        self._task_id_to_task[task.task_id] = task
+        await self._unassigned.put(task.task_id)
 
     async def on_task_reroute(self, task_id: bytes):
         assert task_id in self._task_id_to_client
 
-        client = self._task_id_to_client.pop(task_id)
-        task = self._task_id_to_task.pop(task_id)
         self._running.remove(task_id)
+        await self._unassigned.put(task_id)
 
-        await self._unassigned.put((client, task))
-
-    async def on_task_cancel(self, client: bytes, task_id: bytes):
-        if task_id in self._canceling:
-            logging.warning(f"already canceling: task_id={task_id.hex()}")
-            await self._binder.send(
-                client, MessageType.TaskCancelEcho, TaskCancelEcho(task_id, TaskEchoStatus.Duplicated)
-            )
+    async def on_task_cancel(self, client: bytes, task_cancel: TaskCancel):
+        if task_cancel.task_id in self._unassigned:
+            self._unassigned.remove(task_cancel.task_id)
             return
 
-        if task_id not in self._running:
-            logging.warning(f"cannot cancel task is not running: task_id={task_id.hex()}")
-            await self._binder.send(
-                client, MessageType.TaskCancelEcho, TaskCancelEcho(task_id, TaskEchoStatus.Duplicated)
-            )
+        if task_cancel.task_id not in self._running:
+            logging.warning(f"cannot cancel task is not running: task_id={task_cancel.task_id.hex()}")
             return
 
-        self._running.remove(task_id)
-        self._canceling.add(task_id)
-        await self._binder.send(client, MessageType.TaskCancelEcho, TaskCancelEcho(task_id, TaskEchoStatus.CancelOK))
-        await self._worker_manager.on_task_cancel(task_id)
+        await self._worker_manager.on_task_cancel(client, task_cancel)
+
+    async def on_task_cancel_echo(self, worker: bytes, task_cancel_echo: TaskCancelEcho):
+        if task_cancel_echo.task_id not in self._running:
+            logging.warning(f"received cancel echo that is not canceling state: task_id={task_cancel_echo.task_id}")
+            return
+
+        self._running.remove(task_cancel_echo.task_id)
+        task = self._task_id_to_task.pop(task_cancel_echo.task_id)
+        self._task_id_to_client.pop(task_cancel_echo.task_id)
+        await self._function_manager.on_task_done_function(task.task_id, task.function_id)
+        self._canceled_count += 1
 
     async def on_task_done(self, result: TaskResult):
         """job done can be success or failed"""
         if result.status == TaskStatus.Success:
-            await self.__on_task_success(result)
+            await self.__on_task_done(result)
+            self._success_count += 1
             return
 
         if result.status == TaskStatus.Failed:
-            await self.__on_task_failed(self._running, "fail", result)
-            return
-
-        if result.status == TaskStatus.Canceled:
-            await self.__on_task_failed(self._canceling, "cancel", result)
+            await self.__on_task_done(result)
+            self._failed_count += 1
             return
 
         raise ValueError(f"unknown TaskResult status: {result.status}")
 
-    async def __routine(self):
-        client, task = await self._unassigned.get()
-
-        if not await self._worker_manager.assign_task_to_worker(task):
-            await self._unassigned.put((client, task))
-            return
-
-        self._task_id_to_client[task.task_id] = client
-        self._task_id_to_task[task.task_id] = task
-        self._running.add(task.task_id)
-
-    async def __on_task_success(self, result: TaskResult):
+    async def __on_task_done(self, result: TaskResult):
         if result.task_id not in self._running:
             return
 
@@ -137,21 +128,3 @@ class VanillaTaskManager(TaskManager, Looper):
 
         await self._binder.send(client, MessageType.TaskResult, result)
         await self._function_manager.on_task_done_function(task.task_id, task.function_id)
-
-    async def __on_task_failed(
-        self, processing_set: Set[bytes], fail_type: Literal["cancel", "fail"], result: TaskResult
-    ):
-        if result.task_id not in processing_set:
-            return
-
-        processing_set.remove(result.task_id)
-        task = self._task_id_to_task.pop(result.task_id)
-        client = self._task_id_to_client.pop(result.task_id)
-
-        if fail_type == "cancel":
-            self._canceled_count += 1
-        elif fail_type == "fail":
-            self._failed_count += 1
-
-        await self._binder.send(client, MessageType.TaskResult, result)
-        await self._function_manager.on_task_done_function(task.task_id, task.function_id)
