@@ -1,10 +1,8 @@
-import asyncio
 import logging
 import time
 from typing import Dict, List, Optional, Tuple
 
 from scaled.io.async_binder import AsyncBinder
-from scaled.io.config import CLEANUP_INTERVAL_SECONDS
 from scaled.scheduler.mixins import Looper, TaskManager, WorkerManager
 from scaled.protocol.python.message import (
     BalanceRequest,
@@ -12,6 +10,8 @@ from scaled.protocol.python.message import (
     Heartbeat,
     MessageType,
     Task,
+    TaskCancelEcho,
+    TaskEchoStatus,
     TaskResult,
     TaskCancel,
 )
@@ -21,9 +21,16 @@ POLLING_TIME = 1
 
 
 class VanillaWorkerManager(WorkerManager, Looper):
-    def __init__(self, per_worker_queue_size: int, timeout_seconds: int, load_balance_seconds: int):
+    def __init__(
+        self,
+        per_worker_queue_size: int,
+        timeout_seconds: int,
+        load_balance_seconds: int,
+        load_balance_trigger_times: int,
+    ):
         self._timeout_seconds = timeout_seconds
         self._load_balance_seconds = load_balance_seconds
+        self._load_balance_trigger_times = load_balance_trigger_times
 
         self._binder: Optional[AsyncBinder] = None
         self._task_manager: Optional[TaskManager] = None
@@ -32,6 +39,7 @@ class VanillaWorkerManager(WorkerManager, Looper):
         self._allocator = QueuedAllocator(per_worker_queue_size)
 
         self._last_balance_advice = None
+        self._load_balance_advice_same_count = 0
 
     def hook(self, binder: AsyncBinder, task_manager: TaskManager):
         self._binder = binder
@@ -52,20 +60,35 @@ class VanillaWorkerManager(WorkerManager, Looper):
         await self._binder.send(worker, MessageType.Task, task)
         return True
 
-    async def on_task_cancel(self, task_id: bytes):
-        worker = self._allocator.get_assigned_worker(task_id)
+    async def on_task_cancel(self, client: bytes, task_cancel: TaskCancel):
+        worker = self._allocator.get_assigned_worker(task_cancel.task_id)
         if worker is None:
-            logging.error(f"cannot find task_id={task_id.hex()} in task workers")
+            logging.error(f"cannot find task_id={task_cancel.task_id.hex()} in task workers")
             return
 
-        await self._binder.send(worker, MessageType.TaskCancel, TaskCancel(task_id))
+        await self._binder.send(worker, MessageType.TaskCancel, TaskCancel(task_cancel.task_id))
+
+    async def on_task_cancel_echo(self, worker: bytes, task_cancel_echo: TaskCancelEcho):
+        if task_cancel_echo.status == TaskEchoStatus.CancelFailed:
+            logging.warning(f"cancel task task_id={task_cancel_echo.task_id.hex()} failed")
+            return
+
+        assert task_cancel_echo.status == TaskEchoStatus.CancelOK
+        worker = self._allocator.remove_task(task_cancel_echo.task_id)
+        if worker is None:
+            logging.error(
+                f"received TaskCancelEcho for task_id={task_cancel_echo.task_id.hex()} not known to any worker"
+            )
+            return
+
+        await self._task_manager.on_task_cancel_echo(worker, task_cancel_echo)
 
     async def on_task_done(self, task_result: TaskResult):
         worker = self._allocator.remove_task(task_result.task_id)
         if worker is None:
             logging.error(
                 f"received task_id={task_result.task_id.hex()} not known to any worker, might due to worker get "
-                f"disconnected"
+                f"disconnected or canceled"
             )
             return
 
@@ -85,12 +108,9 @@ class VanillaWorkerManager(WorkerManager, Looper):
     async def has_available_worker(self) -> bool:
         return self._allocator.has_available_worker()
 
-    async def loop(self):
-        logging.info(f"{self.__class__.__name__}: started")
-        while True:
-            await self.__balance_request()
-            await self.__clean_workers()
-            await asyncio.sleep(CLEANUP_INTERVAL_SECONDS)
+    async def routine(self):
+        await self.__balance_request()
+        await self.__clean_workers()
 
     async def statistics(self) -> Dict:
         worker_to_task_numbers = self._allocator.statistics()
@@ -113,15 +133,21 @@ class VanillaWorkerManager(WorkerManager, Looper):
         if self._load_balance_seconds <= 0:
             return
 
-        if self._last_balance_advice is None:
-            self._last_balance_advice = self._allocator.balance()
-            return
-
         current_advice = self._allocator.balance()
-        if current_advice != self._last_balance_advice:
+        if self._last_balance_advice != current_advice:
             self._last_balance_advice = current_advice
+            self._load_balance_advice_same_count = 1
             return
 
+        self._load_balance_advice_same_count += 1
+        if self._load_balance_advice_same_count != self._load_balance_trigger_times:
+            return
+
+        if not current_advice:
+            return
+
+        logging.info(f"balance: {current_advice}")
+        self._last_balance_advice = current_advice
         for worker, number_of_tasks in current_advice.items():
             await self._binder.send(worker, MessageType.BalanceRequest, BalanceRequest(number_of_tasks))
 
